# uvit9pt.py
#
# Script to generate coordinates for a 9-point pattern
# to be traversed in order to cover an extended field 
# around a target for which UVIT clearance is required.
#
# User needs to input the name, Right Ascension and Declination
# of the target.
#
# The computed 9-point pattern is written into an output text file
# truncation/roundoff errors are within one arc sec
#
# usage: python ./uvit9pt.py
#
# Dipankar Bhattacharya 11 January 2018
#
# $Id: uvit9pt.py,v 1.5 2018/01/14 03:37:52 dipankar Exp $ 
# $Log: uvit9pt.py,v $
# Revision 1.5  2018/01/14 03:37:52  dipankar
# code cleanup
#
# Revision 1.4  2018/01/13 10:24:08  dipankar
# Fixed a bug in input parsing of negative zero deg dec
#
# Revision 1.3  2018/01/12 07:57:23  dipankar
# Updated input prompts
#
# Revision 1.2  2018/01/12 07:20:09  dipankar
# included carry over management for output coordinates
#
#
from math import *

# procedure to convert deg, min, sec format input to decimal degree
def dmsconv(deg,min,sec):
  if deg < 0:
     sign=-1
  else:
     sign=+1
  absdeg=sign*deg
  degf=absdeg+(float(min)+sec/60.0)/60.0
  degf=sign*degf
  return degf

# multiplication by dr converts degree to radian
dr=pi/180.0

# xoff, yoff defines the normalised traverse pattern
# the coordinate offsets are on a 3x3 grid, xoff, yoff
# are the corresponding grid indices
xoff=(0,1,-1,1,-1,1,0,0,-1)
yoff=(0,-1,0,0,1,1,-1,1,-1)

# gridstep is the grid spacing in radians 
gridstep=0.11*dr

# twist is the orientation of the grid w.r.t. the local meridian
twist=0.0

# receive user inputs
target=raw_input("Target Name: ")
rastr=raw_input("RA  [J2000] (hh mm ss.ss): ")
decstr=raw_input("Dec [J2000] (+dd mm ss.ss): ")

# convert input RA to decimal degree
rra=rastr.split()
hour=int(rra[0])
hour=abs(hour)
min=int(rra[1])
min=abs(min)
if len(rra) < 3 :
   sec=0.0
else:
   sec=float(rra[2])
sec=abs(sec)
RAsrc=15.0*dmsconv(hour,min,sec)

# convert input Dec to decimal degree
sign=1
rdec=decstr.split()
deg=int(rdec[0])
min=int(rdec[1])
min=abs(min)
if len(rdec) < 3 :
   sec=0.0
else:
   sec=float(rdec[2])
sec=abs(sec)
Decsrc=dmsconv(deg,min,sec)
if deg==0 :
   if '-' in rdec[0] :
      Decsrc=-Decsrc

# pre-compute some required trigonometric coefficients
ca=cos(RAsrc*dr)
sa=sin(RAsrc*dr)
cd=cos(Decsrc*dr)
sd=sin(Decsrc*dr)
ct=cos(twist*dr)
st=sin(twist*dr)

# create an output file named after the target and open it
outname=target.strip()+"_9pt.txt"
outfile=open(outname,'wb')

# compute and write the coordinates of the 9-point pattern
for i in range(0, 9):
    thetax=xoff[i]*gridstep
    thetay=yoff[i]*gridstep
    tx=tan(thetax)
    ty=tan(thetay)
    vz=1.0/sqrt(1.0+tx*tx+ty*ty)
    vx=vz*tx
    vy=vz*ty
    xm=vx*ct-vy*st
    ym=vy*ct+vx*st
    zs=vz*sd+ym*cd
    xs=(vz*cd-ym*sd)*ca+xm*sa
    ys=(vz*cd-ym*sd)*sa-xm*ca
    dec=asin(zs)/dr
    ra=atan2(ys,xs)/dr
    if ra < 0.0:
       ra=ra+360.0
    if ra > 360.0:
       ra=ra+360.0

# output formatting of RA to dms format
    ra=ra/15.0
    rah=int(ra)
    ram=int(60.0*(ra-rah))
    ras=3600.0*(ra-rah-(1.0*ram)/60.0)
    rasi=int(ras)
    rasf=int(100.0*(ras-rasi)+0.5)

#  roundoff carry over mamagement
    if rasf > 99:
       rasi=rasi+1
       rasf=rasf-100
    if rasi > 59:
       ram=ram+1
       rasi=rasi-60
    if ram > 59:
       rah=rah+1
       ram=ram-60
    if rah > 24:
       rah=24

# output formatting of Dec to signed dms format
    if dec < 0.0:
       sgc="-"
    else:
       sgc="+"
    dec=abs(dec)
    decd=int(dec)
    decm=int(60.0*(dec-decd))
    decs=3600.0*(dec-decd-(1.0*decm)/60.0)
    decsi=int(decs)
    decsf=int(100.0*(decs-decsi)+0.5)

#  roundoff carry over mamagement
    if decsf > 99:
       decsi=decsi+1
       decsf=decsf-100
    if decsi > 59:
       decm=decm+1
       decsi=decsi-60
    if decm > 59:
       decd=decd+1
       decm=decm-60
    if decd > 90:
       decd=90

    j=i+1
    outfile.write(target.strip()+"_"+str(j)+"  %2.2d %2.2d %2.2d.%2.2d  %1s%2.2d %2.2d %2.2d.%2.2d\n" %(rah,ram,rasi,rasf,sgc,decd,decm,decsi,decsf))

print "Nine-point targets written in the file "+outname
